package com.bwie.auth.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.bwie.auth.feign.UserFeignService;
import com.bwie.auth.service.AuthService;
import com.bwie.common.constants.JwtConstants;
import com.bwie.common.constants.TokenConstants;
import com.bwie.common.domain.User;
import com.bwie.common.domain.request.UserRequest;
import com.bwie.common.domain.response.JwtResponse;
import com.bwie.common.result.Result;
import com.bwie.common.utils.JwtUtils;
import com.bwie.common.utils.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;
import java.util.concurrent.TimeUnit;

/**
 * @Author LiYonJie
 * @Date 2023/12/21
 * @Description TODO
 * @Version 1.0
 */
@Service
public class AuthServiceImpl implements AuthService {

    @Autowired
    private UserFeignService userFeignService;

    @Autowired
    private RedisTemplate<String, String> redisTemplate;

    @Autowired
    private HttpServletRequest request;


    @Override
    public Result<JwtResponse> login(UserRequest userRequest) {
        if (StringUtils.isEmpty(userRequest.getTel()) || StringUtils.isEmpty(userRequest.getPassword())){
            return Result.error("不能为空");
        }
        Result<User> byTel = userFeignService.findByTel(userRequest.getTel());
        User loginUser = byTel.getData();
        if (null == loginUser){
            return Result.error("该用户不存在");
        }
        if (!loginUser.getPassword().equals(loginUser.getPassword())){
            return Result.error("密码错误");
        }
        // 1.3 使用JWT完成登录认证（2分）
        String userKey = UUID.randomUUID().toString().replaceAll("-", "");
        redisTemplate.opsForValue().set(TokenConstants.LOGIN_TOKEN_KEY + userKey, JSONObject.toJSONString(loginUser),
                30, TimeUnit.MINUTES);
        Map<String, Object> jwtMap = new HashMap<>(100, 100);
        jwtMap.put(JwtConstants.USER_KEY, userKey);
        jwtMap.put(JwtConstants.DETAILS_USER_ID, loginUser.getUserId());
        String token = JwtUtils.createToken(jwtMap);

        JwtResponse jwtResponse = new JwtResponse();
        jwtResponse.setToken(token);
        jwtResponse.setExpireTime("30MIN");
        return Result.success(jwtResponse);
    }

    @Override
    public User userInfo() {
        String token = request.getHeader(TokenConstants.TOKEN);
        String userKey = JwtUtils.getUserKey(token);
        String json = redisTemplate.opsForValue().get(TokenConstants.LOGIN_TOKEN_KEY + userKey);
        return JSONObject.parseObject(json, User.class);
    }

    @Override
    public void logout() {
        String token = request.getHeader(TokenConstants.TOKEN);
        String userKey = JwtUtils.getUserKey(token);
        redisTemplate.delete(TokenConstants.LOGIN_TOKEN_KEY + userKey);
    }
}
