package com.bwie.es.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.bwie.common.domain.Friends;
import com.bwie.common.domain.request.FriendsRequest;
import com.bwie.common.domain.response.FriendsResponse;
import com.bwie.common.result.PageResult;
import com.bwie.common.result.Result;
import com.bwie.common.utils.StringUtils;
import com.bwie.es.service.EsService;
import org.elasticsearch.action.index.IndexRequest;
import org.elasticsearch.action.search.SearchRequest;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.common.document.DocumentField;
import org.elasticsearch.common.text.Text;
import org.elasticsearch.common.xcontent.XContentType;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.search.SearchHit;
import org.elasticsearch.search.SearchHits;
import org.elasticsearch.search.builder.SearchSourceBuilder;
import org.elasticsearch.search.fetch.subphase.highlight.HighlightBuilder;
import org.elasticsearch.search.fetch.subphase.highlight.HighlightField;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Map;

/**
 * @Author LiYonJie
 * @Date 2023/12/21
 * @Description TODO
 * @Version 1.0
 */
@Service
public class EsServiceImpl implements EsService {

    @Autowired
    private RestHighLevelClient restHighLevelClient;

    public static final String INDEX_NAME = "friend";

    @Override
    public Result<PageResult<FriendsResponse>> search(FriendsRequest friendRequest) {
        long total = 0;
        ArrayList<FriendsResponse> list = new ArrayList<>();

        try {
            SearchRequest searchRequest = new SearchRequest(INDEX_NAME);
            SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
            BoolQueryBuilder boolQuery = QueryBuilders.boolQuery();
            if (StringUtils.isEmpty(friendRequest.getUsername())){
               boolQuery.must(QueryBuilders.matchQuery("username",friendRequest.getUsername()));
            }
            if (null != friendRequest.getStatues()){
                boolQuery.must(QueryBuilders.rangeQuery("statues"));
            }
            searchSourceBuilder.query(boolQuery);
            HighlightBuilder highlightBuilder = new HighlightBuilder();
            highlightBuilder.field("username");

            highlightBuilder.preTags("<font style=\"color:red\">");
            highlightBuilder.postTags("</font>");

            searchSourceBuilder.highlighter(highlightBuilder);
            searchRequest.source(searchSourceBuilder);

            SearchResponse search = restHighLevelClient.search(searchRequest, RequestOptions.DEFAULT);
            SearchHits searchHits = search.getHits();
            total = searchHits.getTotalHits().value;
            SearchHit[] hits = searchHits.getHits();
            for (SearchHit hit : hits) {
                String sourceAsString = hit.getSourceAsString();
                FriendsResponse friendsResponse = JSONObject.parseObject(sourceAsString, FriendsResponse.class);
                Map<String, HighlightField> fields = hit.getHighlightFields();
                if (null != fields){
                    HighlightField field = fields.get("username");
                    if (null != field){
                        String str = "";
                        Text[] fragments = field.fragments();
                        for (Text fragment : fragments) {
                            str += fragment;
                        }
                        friendsResponse.setUsername(str);
                    }
                    list.add(friendsResponse);
                    friendsResponse.setUserId(Integer.valueOf(hit.getId()));
                }
            }
            return PageResult.toResult(total, list);
        } catch (IOException e) {
            e.printStackTrace();
        }
        return null;
    }

    @Override
    public void add(FriendsResponse friends) {
        try {
            IndexRequest indexRequest = new IndexRequest(INDEX_NAME);
            indexRequest.id(friends.getUserId().toString());
            indexRequest.source(JSONObject.toJSONString(friends), XContentType.JSON);
            restHighLevelClient.index(indexRequest,RequestOptions.DEFAULT);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}
