package com.bwie.user.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.bwie.common.constants.TokenConstants;
import com.bwie.common.domain.User;
import com.bwie.common.domain.request.FriendsRequest;
import com.bwie.common.domain.response.FriendsResponse;
import com.bwie.common.result.PageResult;
import com.bwie.common.result.Result;
import com.bwie.common.utils.JwtUtils;
import com.bwie.user.mapper.FriendMapper;
import com.bwie.user.service.FriendService;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

/**
 * @Author LiYonJie
 * @Date 2023/12/21
 * @Description TODO
 * @Version 1.0
 */
@Service
public class FriendServiceImpl implements FriendService {

    @Autowired
    private FriendMapper friendMapper;

    @Autowired
    private RedisTemplate<String, String> redisTemplate;

    @Autowired
    private HttpServletRequest request;

    /**
     * 1.4 分页查询数据成功
     *      列表数据根累积总能量进行排序
     * @param friendRequest
     * @return
     */
    @Override
    public Result<PageResult<FriendsResponse>> list(FriendsRequest friendRequest) {
        PageHelper.startPage(friendRequest.getPageNum(), friendRequest.getPageSize());
        friendRequest.setUserId(userInfo().getUserId());
        List<FriendsResponse> list = friendMapper.list(friendRequest);
        PageInfo<FriendsResponse> info = new PageInfo<>(list);
        return PageResult.toResult(info.getTotal(), list);
    }

    /**
     * 1.8 好友申请列表数据展示正确（5分）
     * @return
     */
    @Override
    public List<FriendsResponse> unFriends() {
        FriendsRequest friendsRequest = new FriendsRequest();
        friendsRequest.setUserId(userInfo().getUserId());
        return friendMapper.unFriends(friendsRequest);
    }

    @Override
    public List<FriendsResponse> findAll() {
        return friendMapper.findAll();
    }

    public User userInfo() {
        String token = request.getHeader(TokenConstants.TOKEN);
        String userKey = JwtUtils.getUserKey(token);
        String json = redisTemplate.opsForValue().get(TokenConstants.LOGIN_TOKEN_KEY + userKey);
        return JSONObject.parseObject(json, User.class);
    }
}
